;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Balloon Gliders
;  by Mandew
;
; Balloon Gliders will attach to Mario as he passes by them.
; Mario will glide upwards or downwards while using them.
; Press B or release Y to let 'em go.
;
; Uses Extra Bit: YES
;  Changes between 1-balloon glider and 2-balloon glider.
;   1-balloon glides downwards, 2-balloon glides upwards.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;  Defines

; Customizables
!DipSpeed = #$38
!MaxDownSpeed = #$0C
!MaxUpSpeed = #$F0

; Sprite Tables

!CustomSpriteNumber = $7FAB9E
!CustomSpriteProperties = $7FAB10
!BehaviorState = $C2
!SpriteXPosLo = $E4
!SpriteXPosHi = $14E0
!SpriteXSpeed = $B6
!SpriteYPosLo = $D8
!SpriteYPosHi = $14D4
!SpriteYSpeed = $AA
!SpriteGFXProp = $15F6
!SpriteOffScreenVertical = $186C
!SpriteStatus = $14C8

; OAM
!OAM_X = $0300
!OAM_Y = $0301
!OAM_Tile = $0302
!OAM_Properties = $0303

; Mario

!MarioXPosLo = $94
!MarioXPosHi = $95
!MarioXSpeed = $7B
!MarioYPosLo = $96
!MarioYPosHi = $97
!MarioYSpeed = $7D
!MarioHoldingObject = $148F
!MarioAir = $72
!MarioSpinJump = $140D

; RAM Addresses
!LockFlag = $9D
!IOHeldButtons = $15
!IOPressedButtons = $16
!IOPressedButtons2 = $18
!SoundFXA = $1DFA
!SoundFXB = $1DFC
!EffectiveCounter = $14
!SpringBoarding = $1406

; Routines

!JumpPointer = $0086DF
!GetMarioClipping = $03B664
!GetSpriteClipping = $03B69F
!CheckForCollisions = $03B72B
!UpdateXPos = $018022
!UpdateYPos = $01801A
!FinishOAMWrite = $01B7B3

; Constants

!AndRunButton = #$40
!JumpSound = #$01
!SpinJumpSound = #$04
!SpriteLoopInit = #$0C
!AndIsCustomSprite = #$02
!ExtraBits = #$04

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Wrappers
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	print "INIT ",pc                                 
		RTL

	print "MAIN ",pc                                 
		PHB
		PHK
		PLB
		JSR MainRoutine
		PLB
		RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; MainRoutine
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	Return_Main:
		RTS

	MainRoutine:
		JSR DrawGraphics
		JSR SUB_OFF_SCREEN_X0

; GameState Check
		LDA !SpriteStatus,x
		CMP #$08
		BNE Return_Main
		LDA !LockFlag
		BNE Return_Main ; Don't proceed if sprites are locked.

; Behavior Check
		LDA !BehaviorState,x
		JSL !JumpPointer

	BehaviorPointer:
		dw BehaviorIdle
		dw BehaviorGlide
		dw BehaviorAway

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; BehaviorIdle
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	Return_Idle:
		RTS

	BehaviorIdle:
		JSL !GetMarioClipping
		JSL !GetSpriteClipping	
		JSL !CheckForCollisions
		BCC Return_Idle	   ; Don't Proceed unless there is a collision.

		JSR Sub_CancelActiveGliders
		JSR Sub_ActivateThisGlider
		
		RTS

; *************************

Sub_CancelActiveGliders:
	LDA !CustomSpriteNumber,x
	STA $00

	PHX
	LDX !SpriteLoopInit 	;Initiate a Sprite Loop

KeepLooping_A:
	DEX
	CPX #$00
	BMI StopLooping_A	; Don't proceed and stop looping if all the sprites were checked.

	LDA !CustomSpriteProperties,x
	AND !AndIsCustomSprite
	BNE KeepLooping_A	; Don't proceed unless sprite is custom; If not custom, go to KeepLooping_A

	LDA !CustomSpriteNumber,x
	CMP $00
	BNE KeepLooping_A	; Don't proceed unless the sprite's number is the same as this sprite's number.

	LDA !BehaviorState,x
	CMP #$01
	BNE KeepLooping_A	; Don't proceed unless the sprite is on BehaviorState #$01

	INC !BehaviorState,x	
	BRA KeepLooping_A	; Keep on looping.

StopLooping_A:
	PLX
	RTS

; *************************

Sub_ActivateThisGlider:
	STZ !MarioSpinJump
	INC !BehaviorState,x

	LDA !MarioYSpeed
	BMI NoDipSpeed
	CMP !DipSpeed
	BMI NoDipSpeed
	LDA !DipSpeed
	STA !SpriteYSpeed,x
	STA !MarioYSpeed

NoDipSpeed:
	LDA !SpriteYPosLo,x
	SEC
	SBC #$10
	STA !MarioYPosLo
	LDA !SpriteYPosHi,x
	SBC #$00
	STA !MarioYPosHi
	RTS

; *************************

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; BehaviorGlide
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

BehaviorGlide:
	JSR Sub_CheckIfStillRiding
	BCC CancelThisGlider
	BRA YouAreGliding

;***

CancelThisGlider:
	LDA !MarioXSpeed
	STA !SpriteXSpeed,x
	LDA !IOPressedButtons
	BMI MarioJumpsOff

	LDA !IOPressedButtons2
	BMI MarioSpinJumpsOff

MarioDropsOff:
	;STZ !MarioYSpeed	
	;INC !BehaviorState,x
	RTS
		
MarioSpinJumpsOff:
	LDA #$04
	STA !SoundFXB
	LDA #$01
	STA !MarioSpinJump
	BRA OffThisThing

MarioJumpsOff:
	LDA !JumpSound
	STA !SoundFXA

OffThisThing:
	LDA #$0B
	STA !MarioAir
	LDA #$C0
	STA !MarioYSpeed
	INC !BehaviorState,x
	RTS

;***

YouAreGliding:
	LDA #$80
	STA !SpringBoarding

	LDA !MarioXPosLo
	STA !SpriteXPosLo,x
	LDA !MarioXPosHi
	STA !SpriteXPosHi,x  ; Let the sprite copy Mario's X Position

	LDA !MarioYPosLo
	STA !SpriteYPosLo,x
	LDA !MarioYPosHi
	STA !SpriteYPosHi,x  ; Let the sprite copy Mario's Y Position

	LDA !MarioYSpeed
	STA !SpriteYSpeed,x

	LDA !CustomSpriteProperties,x
	AND !ExtraBits
	BNE BalloonGlide2   ; If it is a 2-balloon glider, go to 2BalloonGlide

BalloonGlide1:
	LDA !SpriteYSpeed,x
	BMI TooMuchSpeed1
	CMP !MaxDownSpeed
	BMI TooMuchSpeed1	

	JSR SuperBuyo
TooMuchSpeed1:
	STA !SpriteYSpeed,x
	STA !MarioYSpeed
	RTS

BalloonGlide2:
	LDA !SpriteYSpeed,x
	CMP !MaxUpSpeed
	BMI TooMuchSpeed2	

	JSR SuperBuyo

TooMuchSpeed2:
	STA !SpriteYSpeed,x
	STA !MarioYSpeed
	RTS

DATA_UpdraftForce:
db $0A,$09,$08,$08,$07,$07,$06,$06
db $00,$00,$06,$06,$08,$08,$0A,$0C

SuperBuyo:
	LDA !MarioYSpeed
	PHA
	LSR
	LSR
	LSR
	LSR
	TAY
	PLA

	SEC
	SBC DATA_UpdraftForce,y
	STA !SpriteYSpeed,x
	STA !MarioYSpeed

	;LDA !SpriteYSpeed,x
	;SEC
	;SBC #$08
	;STA !SpriteYSpeed,x
	;STA !MarioYSpeed
	RTS

; *************************

Sub_CheckIfStillRiding:
	LDA !IOPressedButtons
	BMI ClearCarry
	
	LDA !IOPressedButtons2
	BMI ClearCarry

	;LDA !IOHeldButtons
	;AND !AndRunButton
	;BEQ ClearCarry

	SEC
	RTS
ClearCarry:
	CLC
	RTS

; *************************

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; BehaviorAway
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


BehaviorAway:
	LDA !SpriteYSpeed,x
	DEC
	DEC
	CMP #$A0
	BPL NotSpeedLimit3

	LDA #$A0

NotSpeedLimit3:
	STA !SpriteYSpeed,x

	JSL !UpdateXPos
	JSL !UpdateYPos
	RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; DrawGraphics
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

GFX_XDisp: db $00,$00,$F4,$0C
	   db $00,$01,$F3,$0D
	   db $00,$00,$F4,$0C
	   db $00,$FF,$F5,$0B

GFX_YDisp: db $00,$E4,$E8,$E6
	   db $01,$E3,$E8,$E6
	   db $00,$E4,$E7,$E6
	   db $FF,$E4,$E8,$E5

GFX_Tile: db $00,$02,$02,$02
GFX_Property: db $04,$08,$06,$0A

DrawGraphics:
	JSR GET_DRAW_INFO

	LDA !EffectiveCounter	
	AND #$30
	LSR
	LSR
	STA $05

	LDA !SpriteGFXProp,x
	STA $04

	LDA !CustomSpriteProperties,x
	AND !ExtraBits
	BNE GFX_Entry2

GFX_Entry1:
	PHX
	LDA #$02
	TAX
	DEC
	STA $03
	BRA GraphicsLoop_A

GFX_Entry2:
	PHX
	LDA #$04
	TAX
	DEC
	STA $03

GraphicsLoop_A:
	DEX
	BMI StopGraphicsLoop_A

	PHX

	TXA
	CLC
	ADC $05
	TAX

	LDA $00
	CLC
	ADC GFX_XDisp,x
	STA !OAM_X,y

	LDA $01
	CLC
	ADC GFX_YDisp,x
	STA !OAM_Y,y
	
	PLX

	LDA GFX_Tile,x
	STA !OAM_Tile,y
	
	LDA $04
	AND #$01
	ORA GFX_Property,x
	ORA $64
	STA !OAM_Properties,y
	INY
	INY
	INY
	INY
	BRA GraphicsLoop_A

StopGraphicsLoop_A:
	PLX
	LDA $03              
        LDY #$02  
	JSL !FinishOAMWrite	
	RTS

;------------------------------------------------------------------------------------------
; Below are generic subroutines.
;------------------------------------------------------------------------------------------

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Get Draw Info
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TABLE1:             db $0C,$1C
TABLE2:             db $01,$02

GET_DRAW_INFO:      STZ $186C,x             ; reset sprite offscreen flag, vertical
                    STZ $15A0,x             ; reset sprite offscreen flag, horizontal
                    LDA $E4,x               ; \
                    CMP $1A                 ;  | set horizontal offscreen if necessary
                    LDA $14E0,x             ;  |
                    SBC $1B                 ;  |
                    BEQ ON_SCREEN_X         ;  |
                    INC $15A0,x             ; /

ON_SCREEN_X:        LDA $14E0,x             ; \
                    XBA                     ;  |
                    LDA $E4,x               ;  |
                    REP #$20                ;  |
                    SEC                     ;  |
                    SBC $1A                 ;  | mark sprite invalid if far enough off screen
                    CLC                     ;  |
                    ADC #$0040            ;  |
                    CMP #$0180            ;  |
                    SEP #$20                ;  |
                    ROL A                   ;  |
                    AND #$01                ;  |
                    STA $15C4,x             ; / 
                    BNE INVALID             ; 
                    
                    LDY #$00                ; \ set up loop:
                    LDA $1662,x             ;  | 
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice
                    BEQ ON_SCREEN_LOOP      ;  | else, go through loop once
                    INY                     ; / 
ON_SCREEN_LOOP:     LDA $D8,x               ; \ 
                    CLC                     ;  | set vertical offscreen if necessary
                    ADC TABLE1,y            ;  |
                    PHP                     ;  |
                    CMP $1C                 ;  | (vert screen boundry)
                    ROL $00                 ;  |
                    PLP                     ;  |
                    LDA $14D4,x             ;  | 
                    ADC #$00                ;  |
                    LSR $00                 ;  |
                    SBC $1D                 ;  |
                    BEQ ON_SCREEN_Y         ;  |
                    LDA $186C,x             ;  | (vert offscreen)
                    ORA TABLE2,y            ;  |
                    STA $186C,x             ;  |
ON_SCREEN_Y:        DEY                     ;  |
                    BPL ON_SCREEN_LOOP      ; /

                    LDY $15EA,x             ; get offset to sprite OAM
                    LDA $E4,x               ; \ 
                    SEC                     ;  | 
                    SBC $1A                 ;  | $00 = sprite x position relative to screen boarder
                    STA $00                 ; / 
                    LDA $D8,x               ; \ 
                    SEC                     ;  | 
                    SBC $1C                 ;  | $01 = sprite y position relative to screen boarder
                    STA $01                 ; / 
                    RTS                     ; return

INVALID:            PLA                     ; \ return from *main gfx routine* subroutine...
                    PLA                     ;  |    ...(not just this subroutine)
                    RTS                     ; /

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Off-screen
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

                    ;org $03B83B             

TABLE3:              db $40,$B0
TABLE6:             db $01,$FF 
TABLE4:              db $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5:              db $01,$FF,$01,$FF,$01,$00,$01,$FF

SUB_OFF_SCREEN_X0:   LDA #$06                ; \ entry point of routine determines value of $03
                    BRA STORE_03            ;  | 
SUB_OFF_SCREEN_X1:   LDA #$04                ;  |
                    BRA STORE_03            ;  |
SUB_OFF_SCREEN_X2:   LDA #$02                ;  |
STORE_03:            STA $03                 ;  |
                    BRA START_SUB           ;  |
SUB_OFF_SCREEN_HB:   STZ $03                 ; /

START_SUB:           JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    ORA $03                 ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry ;A:0101 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0256 VC:090 00 FL:16953
                    CLC                     ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0280 VC:090 00 FL:16953
                    ADC TABLE4,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0294 VC:090 00 FL:16953
                    ROL $00                 ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0326 VC:090 00 FL:16953
                    CMP $E4,x               ;x pos ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0364 VC:090 00 FL:16953
                    PHP                     ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0394 VC:090 00 FL:16953
                    LDA $1B                 ;x boundry hi ;A:01C0 X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizCHC:0416 VC:090 00 FL:16953
                    LSR $00                 ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0440 VC:090 00 FL:16953
                    ADC TABLE5,y            ;A:0100 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdizcHC:0478 VC:090 00 FL:16953
                    PLP                     ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNvMXdizcHC:0510 VC:090 00 FL:16953
                    SBC $14E0,x             ;x pos high ;A:01FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0538 VC:090 00 FL:16953
                    STA $00                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0570 VC:090 00 FL:16953
                    LSR $01                 ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0594 VC:090 00 FL:16953
                    BCC LABEL20             ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0632 VC:090 00 FL:16953
                    EOR #$80                ;A:01FE X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0648 VC:090 00 FL:16953
                    STA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0664 VC:090 00 FL:16953
LABEL20:             LDA $00                 ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0688 VC:090 00 FL:16953
                    BPL RETURN_2            ;A:017E X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0712 VC:090 00 FL:16953
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZCHC:0140 VC:071 00 FL:21152
                    CPY #$FF                ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizCHC:0172 VC:071 00 FL:21152
                    BEQ KILL_SPRITE         ;A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0188 VC:071 00 FL:21152
                    LDA #$00                ; \ mark sprite to come back    A:FF08 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0204 VC:071 00 FL:21152
                    STA $1938,y             ; /                             A:FF00 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdiZcHC:0220 VC:071 00 FL:21152
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_2:            RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                ;A:0227 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0228 VC:112 00 FL:1142
                    STA $01                 ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0244 VC:112 00 FL:1142
                    TAY                     ;A:0201 X:0006 Y:00EC D:0000 DB:03 S:01ED P:envMXdizcHC:0268 VC:112 00 FL:1142
                    LDA $1C                 ;A:0201 X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0282 VC:112 00 FL:1142
                    CLC                     ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0306 VC:112 00 FL:1142
                    ADC TABLE3,y            ;A:02BD X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0320 VC:112 00 FL:1142
                    ROL $00                 ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0352 VC:112 00 FL:1142
                    CMP $D8,x               ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:enVMXdizCHC:0390 VC:112 00 FL:1142
                    PHP                     ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0420 VC:112 00 FL:1142
                    LDA $001D             ;A:026D X:0006 Y:0001 D:0000 DB:03 S:01EC P:eNVMXdizcHC:0442 VC:112 00 FL:1142
                    LSR $00                 ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdiZcHC:0474 VC:112 00 FL:1142
                    ADC TABLE6,y            ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:enVMXdizCHC:0512 VC:112 00 FL:1142
                    PLP                     ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01EC P:envMXdiZCHC:0544 VC:112 00 FL:1142
                    SBC $14D4,x             ;A:0200 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNVMXdizcHC:0572 VC:112 00 FL:1142
                    STA $00                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0604 VC:112 00 FL:1142
                    LDY $01                 ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizcHC:0628 VC:112 00 FL:1142
                    BEQ LABEL22             ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0652 VC:112 00 FL:1142
                    EOR #$80                ;A:02FF X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0668 VC:112 00 FL:1142
                    STA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0684 VC:112 00 FL:1142
LABEL22:             LDA $00                 ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0708 VC:112 00 FL:1142
                    BPL RETURN_2            ;A:027F X:0006 Y:0001 D:0000 DB:03 S:01ED P:envMXdizcHC:0732 VC:112 00 FL:1142
                    BMI ERASE_SPRITE        ;A:0280 X:0006 Y:0001 D:0000 DB:03 S:01ED P:eNvMXdizCHC:0170 VC:064 00 FL:1195

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return